//
//  main.c
//  URL Handling - C
//
//  Created by Sam Marshall on 6/4/14.
//
//

// Usage: ./url-handling-c [url]

#include <CoreFoundation/CoreFoundation.h>

char* __CFStringGetCString(CFStringRef str) {
	if (str) {
		CFIndex alloclen = CFStringGetMaximumSizeForEncoding(CFStringGetLength(str), CFStringGetFastestEncoding(str)) + 1;
		char *cstr = calloc(1, alloclen);
		CFStringGetCString(str, cstr, alloclen, CFStringGetFastestEncoding(str));
		return cstr;
	}
	return "";
}

int main(int argc, const char * argv[]) {
	if (argc == 2) {
		printf("input: %s\n",argv[1]);
		CFStringRef inputString = CFStringCreateWithBytes(kCFAllocatorDefault, (UInt8*)argv[1], strlen(argv[1]), kCFStringEncodingUTF8, false);
		if (CFStringHasPrefix(inputString, CFSTR("http://")) || CFStringHasPrefix(inputString, CFSTR("https://")) || CFStringHasPrefix(inputString, CFSTR("www"))) {
			printf("This is a valid URL.\n");
			CFURLRef inputURL = CFURLCreateWithString(kCFAllocatorDefault, inputString, NULL);
			CFShow(inputURL);
			
			// now we have the URL stored as a CFURL, we want to parse out the root domain and keep track of that
			CFStringRef outputString = CFURLGetString(inputURL);
			CFMutableStringRef mutableString = CFStringCreateMutableCopy(kCFAllocatorDefault, CFStringGetLength(outputString), outputString);
			CFRelease(inputURL);
			if (CFStringHasPrefix(mutableString, CFSTR("http://")) || CFStringHasPrefix(mutableString, CFSTR("https://")) || CFStringHasPrefix(inputString, CFSTR("www."))) {
				CFStringTrim(mutableString, CFSTR("http://"));
				CFStringTrim(mutableString, CFSTR("https://"));
				CFStringTrim(mutableString, CFSTR("www."));
			}
			CFShow(mutableString);
			CFArrayRef urlParts = CFStringCreateArrayBySeparatingStrings(kCFAllocatorDefault, mutableString, CFSTR("/"));
			CFStringRef domainURL = CFArrayGetValueAtIndex(urlParts, 0);
			CFShow(domainURL);
			// code to store domainURL off in our tracking software...
			
			// making it a valid URL again
			if (CFStringGetLength(domainURL)) { // this will always evaluate to true
				CFStringInsert(mutableString, 0, CFSTR("http://"));
			}
			CFRelease(urlParts);
			long stringSize = CFStringGetLength(mutableString) + 6;
			char *open = calloc(1, stringSize);
			snprintf(open, stringSize, "open %s", __CFStringGetCString(mutableString));
			printf("%s\n",open);
			
			/*
			 This is using a call to system() to take a shortcut in replicating the "openURL" functionality provided
				by the custom core library used in the original software. The real code was much more complex but resulted
				in the same resulting call as it was expecting valid web URLs and registed URL-schemes on the host 
				(e.g.: http://, https://, rdar://, feed://, irc://, etc)
			*/
			system(open);
			CFRelease(mutableString);
			free(open);
		} else {
			printf("Please supply a valid URL\n");
		}
		CFRelease(inputString);
	} else {
		printf("Please supply a url.\n");
	}
	return 0;
}